@@ -166,7 +166,9 @@ public:
       // Increment iterators and update length
       ++previous;
       ++pit;
-      CGAL_assertion(pit != points_.end());
+
+      if (pit == points_.end())
+        return *previous;
 
       segment_length = this->distance(*previous,*pit);
     }
@@ -378,30 +380,30 @@ struct Mesh_domain_segment_of_curve_primitive{
 
 template <typename MDwPF, bool patch_id_is_streamable>
 struct Display_incidences_to_patches_aux {
-  template <typename Container>
-  void operator()(std::ostream& os, typename MDwPF::Curve_segment_index id,
+  template <typename Container, typename Point>
+  void operator()(std::ostream& os, Point p, typename MDwPF::Curve_segment_index id,
                   const Container&) const;
 };
 
 template <typename MDwPF> //specialization when patch_id_is_streamable == false
 struct Display_incidences_to_patches_aux<MDwPF, false> {
-  template <typename Container>
-  void operator()(std::ostream& os,
+  template <typename Container, typename Point>
+  void operator()(std::ostream& os, Point p,
                   typename MDwPF::Curve_segment_index id,
                   const Container&) const;
 };
 
 template <typename MDwPF, bool curve_id_is_streamable>
 struct Display_incidences_to_curves_aux {
-  template <typename Container>
-  void operator()(std::ostream& os, typename MDwPF::Curve_segment_index id,
+  template <typename Container, typename Point>
+  void operator()(std::ostream& os, Point p, typename MDwPF::Curve_segment_index id,
                   const Container&) const;
 };
 
 template <typename MDwPF> //specialization when curve_id_is_streamable == false
 struct Display_incidences_to_curves_aux<MDwPF, false> {
-  template <typename Container>
-  void operator()(std::ostream& os, typename MDwPF::Curve_segment_index id,
+  template <typename Container, typename Point>
+  void operator()(std::ostream& os, Point p,  typename MDwPF::Curve_segment_index id,
                   const Container&) const;
 };
 
@@ -478,6 +480,14 @@ public:
     , current_corner_index_(1)
     , current_curve_index_(1)
     , curves_aabb_tree_is_built(false) {}
+
+  template <typename T1, typename T2, typename T3, typename T4>
+  Mesh_domain_with_polyline_features_3(const T1& o1, const T2& o2,
+                                       const T3& o3, const T4& o4)
+    : Base(o1, o2, o3, o4)
+    , current_corner_index_(1)
+    , current_curve_index_(1)
+    , curves_aabb_tree_is_built(false) {}
 #endif
 
   /// Destructor
@@ -559,7 +569,7 @@ public:
   const Surface_patch_index_set&
   get_incidences(Curve_segment_index id) const;
 
-  void display_corner_incidences(std::ostream& os, Corner_index id);
+  void display_corner_incidences(std::ostream& os, Point_3, Corner_index id);
 
   template <typename InputIterator>
   void
@@ -819,13 +829,14 @@ namespace internal { namespace Mesh_3 {
 
 template <typename MDwPF_, bool curve_id_is_streamable>
 // here 'curve_id_is_streamable' is true
-template <typename Container2>
+template <typename Container2, typename Point>
 void
 Display_incidences_to_curves_aux<MDwPF_,curve_id_is_streamable>::
-operator()(std::ostream& os, typename MDwPF_::Curve_segment_index id,
+operator()(std::ostream& os, Point p, typename MDwPF_::Curve_segment_index id,
            const Container2& corners_tmp_incidences_of_id) const
 {
-  os << "Corner #" << id << " is incident to the following curves: {";
+  os << "Corner #" << id << " (" << p
+     << ") is incident to the following curves: {";
   BOOST_FOREACH(typename MDwPF_::Curve_segment_index curve_index,
                 corners_tmp_incidences_of_id)
   {
@@ -836,26 +847,28 @@ operator()(std::ostream& os, typename MDwPF_::Curve_segment_index id,
 
 template <class MDwPF_>
 // here 'curve_id_is_streamable' is false
-template <typename Container2>
+template <typename Container2, typename Point>
 void
 Display_incidences_to_curves_aux<MDwPF_,false>::
-operator()(std::ostream& os, typename MDwPF_::Curve_segment_index id,
+operator()(std::ostream& os, Point p, typename MDwPF_::Curve_segment_index id,
            const Container2& corners_tmp_incidences_of_id) const
 {
-  os << "Corner #" << id << " is incident to "
+  os << "Corner #" << id << " (" << p
+     << ") is incident to "
      << corners_tmp_incidences_of_id .size()
      << " curve(s).\n";
 }
 
 template <typename MDwPF_, bool patch_id_is_streamable>
 // here 'patch_id_is_streamable' is true
-template <typename Container>
+template <typename Container, typename Point>
 void
 Display_incidences_to_patches_aux<MDwPF_,patch_id_is_streamable>::
-operator()(std::ostream& os, typename MDwPF_::Curve_segment_index id,
+operator()(std::ostream& os, Point p, typename MDwPF_::Curve_segment_index id,
            const Container& corners_incidences_of_id) const
 {
-  os << "Corner #" << id << " is incident to the following patches: {";
+  os << "Corner #" << id << " (" << p
+     << ") is incident to the following patches: {";
   BOOST_FOREACH(typename MDwPF_::Surface_patch_index i,
                 corners_incidences_of_id)
   {
@@ -866,13 +879,13 @@ operator()(std::ostream& os, typename MDwPF_::Curve_segment_index id,
 
 template <class MDwPF_>
 // here 'patch_id_is_streamable' is false
-template <typename Container>
+template <typename Container, typename Point>
 void
 Display_incidences_to_patches_aux<MDwPF_,false>::
-operator()(std::ostream& os, typename MDwPF_::Curve_segment_index id,
+operator()(std::ostream& os, Point p, typename MDwPF_::Curve_segment_index id,
            const Container& corners_incidences_id) const
 {
-  os << "Corner #" << id << " is incident to "
+  os << "Corner #" << id << " (" << p << ") is incident to "
      << corners_incidences_id.size()
      << " surface patch(es).\n";
 }
@@ -882,7 +895,7 @@ operator()(std::ostream& os, typename MDwPF_::Curve_segment_index id,
 template <class MD_>
 void
 Mesh_domain_with_polyline_features_3<MD_>::
-display_corner_incidences(std::ostream& os, Corner_index id)
+display_corner_incidences(std::ostream& os, Point_3 p, Corner_index id)
 {
   typedef Mesh_domain_with_polyline_features_3<MD_> Mdwpf;
   typedef is_streamable<Surface_patch_index> i_s_spi;
@@ -891,8 +904,8 @@ display_corner_incidences(std::ostream& os, Corner_index id)
   using namespace internal::Mesh_3;
   typedef Display_incidences_to_curves_aux<Mdwpf,i_s_csi::value> D_i_t_c;
   typedef Display_incidences_to_patches_aux<Mdwpf,i_s_spi::value> D_i_t_p;
-  D_i_t_c()(os, id, corners_tmp_incidences_[id]);
-  D_i_t_p()(os, id, corners_incidences_[id]);
+  D_i_t_c()(os, p, id, corners_tmp_incidences_[id]);
+  D_i_t_p()(os, p, id, corners_incidences_[id]);
 }
 
 template <class MD_>
@@ -930,7 +943,7 @@ compute_corners_incidences()
                                    incidences.begin()));
     }
 #ifdef CGAL_MESH_3_PROTECTION_DEBUG
-    display_corner_incidences(std::cerr, id);
+    display_corner_incidences(std::cerr, cit->first, id);
 #endif // CGAL_MESH_3_PROTECTION_DEBUG
 
     // increment the loop variable
